{# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.
#}
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8" />
  </head>
  <body>
    <ul>
        <li><a href="{{ url_for('index') }}">Home (Open to everyone)</a></li>
        <li><a href="{{ url_for('role_admin') }}">Role admin (Only users with role 'admin') </a></li>
        <li><a href="{{ url_for('action_open') }}">Action open (Only users with action 'open' allowed)</a></li>
        <li><a href="{{ url_for('action_read') }}">Action delete (Only users with action 'read' allowed)</a></li>
    </ul>

    {%- block page_body %}{%- endblock page_body %}
    <p>The current application is using the next actions. The red ones are not allowed yo your current user.</p>
    <ul>
        {% for action in actions.keys() %}
            {% if actions[action] %}
                <li style="color: green">{{action}}</li>
            {% else: %}
                <li style="color: red">{{action}}</li>
            {% endif %}
        {% endfor %}
    </ul>
  </body>
</html>